import fs from "node:fs";
import http from "node:http";
import path from "node:path";
import {fileURLToPath} from "node:url";
// 'open' is built as ESM module, which is not loadable in a CommonJS module
import open from "open";

/**
 * If TypeScript generates 'require' then importing 'open' fails since 'open' is built as ESM module...
 * https://mariusschulz.com/blog/dynamic-import-expressions-in-typescript
 * So, '"module": "ES2020",' *IS REQUIRED*, but all modules are dealt as ESM modules...
 */
// import('../node_modules/open').then(open => {
//     console.info(JSON.stringify(open));
// });

const server = http.createServer((request, response) => {
    if (request.url === "/") { // 'index.html' served...
        response.writeHead(200, {"Content-Type": "text/html; charset=utf-8"});
        const __dirname = path.dirname(fileURLToPath(import.meta.url));
        fs.readFile(path.resolve(__dirname, '..') + '/index.html', null, (error, data) => {
            if (error) {
                response.writeHead(404);
                response.write("Whoops! 'index.html' not found...");
            } else
                response.write(data); // Content of 'index.html'...
            response.end();
        });
    } else
    if (request.url === '/Server_sent_data') { // Client asks for dealing with 'Server-sent' data...
        response.statusCode = 200;
        response.setHeader("Access-Control-Allow-Origin", "*"); // CORS
        response.setHeader("Cache-Control", "no-cache");
        // Client and server agree to keep the connection for subsequent requests or responses open:
        response.setHeader("connection", "keep-alive");
        response.setHeader("Content-Type", "text/event-stream");

        const data = JSON.stringify({integer: Math.floor(Math.random() * 10)}); // [0 - 10[
        // The event-stream format must be respected (https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format):
        setTimeout(() => response.write(`id: ${(new Date()).toLocaleTimeString()}\ndata: ${data}\n\n`), 2500);
    }
});
server.listen(process.env['PORT'] || 8080);

setTimeout(async (information) => {
    console.info(`${information}`);
    await open("http://localhost:" + (process.env['PORT'] || 8080));
}, 1000, "After 1 sec., browser is ready for dealing with 'Server-sent' data...");

